from datetime import datetime,timedelta,timezone

def parse_tz(tzstr):
	sign = -1+(tzstr[0]=="+")*2 # 1 if timezone is positive, -1 if timezone is negative
	tzstr = tzstr[1:]
	hour,minute = tzstr.split(":")
	hour = int(hour)*sign
	minute = int(minute)*sign
	dt = timedelta(hours=hour,minutes=minute)
	return timezone(dt)
	

timeZone = parse_tz("+8:00")
curTime = datetime.now(tz=timeZone)
print(curTime.hour,curTime.minute)


from .search import Scanner
import fire
from rich import print
from pathlib import Path
from dotenv import load_dotenv
import os


class CLI:
    def init_scanner(self):
        try:
            self.scanner = Scanner()
        except KeyError:
            print("[red] API key not found")
            print("Please set up first:")
            self.setup()

    def setup(self):
        SHODAN_API_KEY = input("Please enter your SHODAN API KEY:")
        CLARIFAI_API_KEY = input("Please enter your CLARIFAI API KEY:")
        GEOIP_API_KEY = input("Please enter your GEOIP API KEY:")
        with open(Path(__file__).parent / ".env", "w") as f:
            f.write(f"SHODAN_API_KEY={SHODAN_API_KEY}\n")
            f.write(f"CLARIFAI_API_KEY={CLARIFAI_API_KEY}\n")
            f.write(f"GEOIP_API_KEY={GEOIP_API_KEY}")

    def status(self):
        print("Status [green]OK[/green]!")

    def search(self, preset, check=True, tag=True, store=False, loc=True, debug=False):
        """
        :param preset: string, the type of pre written camera you want. choose from: 1)"webcamXP" 2)"MJPG 3)"yawCam"
        :param check: boolean, indicates whether or not you want to check if the image is completly black or white.
        :param tag: boolean, indicates whether or not you want to generate descriptions for the webcam.
        :param debug: boolean, indicates whether or not you want to print debug info.
        """
        self.init_scanner()
        self.scanner.scan_preset(preset, check, tag, loc, debug=debug)

    def search_custom(
        self,
        camera_type,
        url_scheme="",
        check_empty_url="",
        check_empty=True,
        tag=True,
        loc=True,
        search_q="webcams",
        debug=False
    ):
        """
        :param camera_type: string, the type of camera. this string must appear in the data returned by shodan
        :param url_scheme: string, the format in which the result will be displayed
        :param check_empty_url: string, the format that leads to an image that could be downloaded
        :param check_empty: boolean, indicates whether or not you want to check if the image is completly black or white
        :param tag: boolean, indicates whether or not you want to generate descriptions for the image
        :param search_q: string, the term to search for in shodan
        :param debug: boolean, indicates whether or not you want to print debug info.
        """
        self.init_scanner()
        self.scanner.scan(
            camera_type=camera_type,
            url_scheme=url_scheme,
            check_empty_url=check_empty_url,
            check_empty=check_empty,
            tag=tag,
            search_q=search_q,
            loc=loc,
            debug=debug
        )

    def show_environ(self):
        directory = Path(__file__).parent
        env_path = directory / ".env"
        load_dotenv(override=True, dotenv_path=env_path)
        shodan = os.getenv("SHODAN_API_KEY")
        clarifai = os.getenv("CLARIFAI_API_KEY")
        geoip = os.getenv("GEOIP_API_KEY")
        print(f"shodan api key:{shodan}")
        print(f"clarifai api key: {clarifai}")
        print(f"geoip api key: {geoip}")



import requests
from datetime import datetime, timedelta, timezone
from pycountry import countries
from pathlib import Path
import json


def parse_tz(tzstr):
    sign = -1 + (tzstr[0] == "+") * 2  # 1 if timezone is positive, -1 if timezone is negative
    tzstr = tzstr[1:]
    hour, minute = tzstr.split(":")
    hour = int(hour) * sign
    minute = int(minute) * sign
    dt = timedelta(hours=hour, minutes=minute)
    return timezone(dt)


def get_time(tzstr):
    timeZone = parse_tz(tzstr)
    curTime = datetime.now(tz=timeZone)
    return curTime.hour, curTime.minute

def get_country_name(country):
    return countries.get(alpha_2=country).name

class Locater:
    def __init__(self, api_key):
        self.api_key = api_key
        self.url = "https://geo.ipify.org/api/v1"
        self.directory = Path(__file__).parent
        self.cache_path = self.directory / "geoip_cache.json"
        self.cache = {}
        self.load_cache()
        self.api_cnt = 0

    def load_cache(self):
        if not self.cache_path.is_file():
            with open(self.cache_path, 'w') as f:
                json.dump(dict(), f)
        with open(self.cache_path, "r") as f:
            self.cache = json.load(f)

    def store_cache(self):
        with open(self.cache_path, 'w') as f:
            json.dump(self.cache, f)

    def locate(self, ip):
        if ip in self.cache:
            t = self.cache[ip]
            country = t['country']
            region = t['region']
            timeZone = t['timezone']
            hour, minute = get_time(timeZone)
        else:
            data = dict()
            data['apiKey'] = self.api_key
            data['ipAddress'] = ip
            r = requests.get(self.url, params=data)
            self.api_cnt += 1
            res = r.json()
            country = res['location']['country']
            region = res['location']['region']
            timeZone = res['location']['timezone']
            hour, minute = get_time(timeZone)
            self.cache[ip] = dict()
            self.cache[ip]['country'] = country
            self.cache[ip]['region'] = region
            self.cache[ip]['timezone'] = timeZone
            self.store_cache()
        return get_country_name(country), region, hour, minute


from setuptools import setup, find_packages
import sys
import os

version = sys.version_info[:2]
if version<(3,6):
    print("scan-for-webcams require python version >= 3.6")
    print("{}.{} detected".format(*version))
    sys.exit(-1)
VERSION = "1.4.3"
reqs = ['shodan','requests','rich','pillow','clarifai','halo','fire','python-dotenv','pycountry']

with open("README.md", "r") as fh: 
    long_description = fh.read()

setup(
    name="scan-for-webcams",
    version=VERSION,
    author="Jett Chen",
    author_email="contact@jettchen.me",
    url="https://github.com/JettChenT/scan-for-webcams",
    license="MIT",
    packages=find_packages(),
    install_requires=reqs,
    long_description=long_description, 
    long_description_content_type="text/markdown", 
    package_data={
        'scanforwebcams':['*.json'],
    },
    entry_points= {'console_scripts':[
          'sfw=scanforwebcams.universal:Main']
        }
)



import os
import sys
import shodan
import requests
import socket
import urllib
import json
import random
from PIL import Image, ImageEnhance
from rich import print
from clarifai.rest import ClarifaiApp
from halo import Halo
from dotenv import load_dotenv
from pathlib import Path
from datetime import datetime
from .geoip import Locater

def handle():
    err = sys.exc_info()[0]
    print("[red]ERROR:[/red]")
    print(err)

class Scanner(object):
    def __init__(self):
        socket.setdefaulttimeout(5)
        directory = Path(__file__).parent
        env_path = directory / ".env"
        load_dotenv(override=True, dotenv_path=env_path)
        self.SHODAN_API_KEY = os.getenv("SHODAN_API_KEY")
        if self.SHODAN_API_KEY == None:
            raise KeyError("Shodan API key not found in envrion")
        self.api = shodan.Shodan(self.SHODAN_API_KEY)
        # preset url schemes
        self.clarifai_initialized = False
        self.geoip_initialized = False
        with open(directory / "cams.json") as f:
            self.config = json.load(f)

    def init_clarifai(self):
        self.CLARIFAI_API_KEY = os.getenv("CLARIFAI_API_KEY")
        if self.CLARIFAI_API_KEY == None:
            raise KeyError("Clarifai API key not found in environ")
        self.clarifai_app = ClarifaiApp(api_key=self.CLARIFAI_API_KEY)
        self.clarifai_model = self.clarifai_app.public_models.general_model
        self.clarifai_initialized = True

    def init_geoip(self):
        self.GEOIP_API_KEY = os.getenv("GEOIP_API_KEY")
        if self.GEOIP_API_KEY == None:
            raise KeyError("Geoip API key not found in environ")
        self.locator = Locater(self.GEOIP_API_KEY)
        self.geoip_initialized = True

    def tag_image(self, url):
        response = self.clarifai_model.predict_by_url(url=url)
        results = [
            concept["name"] for concept in response["outputs"][0]["data"]["concepts"]
        ]
        return results

    def check_empty(self, image_source, tolerance=5) -> bool:
        im_loc = ".tmpimage"
        urllib.request.urlretrieve(image_source, im_loc)
        im = Image.open(im_loc)
        extrema = im.convert("L").getextrema()
        if abs(extrema[0] - extrema[1]) <= tolerance:
            return False
        return True

    def scan(
            self,
            camera_type,
            url_scheme="",
            check_empty_url="",
            check_empty=True,
            tag=True,
            geoip=True,
            search_q="webcams",
            debug=False
    ):
        print(f"loc:{geoip}, check_empty:{check_empty}, tag:{tag}")
        if url_scheme == "":
            url_scheme = self.config["default"]["url_scheme"]
        if self.SHODAN_API_KEY == "":
            print("[red]Please set up shodan API key in environ![/red]")
            return
        spinner = Halo(text="Looking for possible servers...", spinner="dots")
        spinner.start()
        if tag and (not self.clarifai_initialized):
            self.init_clarifai()
        if geoip and (not self.geoip_initialized):
            self.init_geoip()
        try:
            results = self.api.search(search_q)
            spinner.succeed("Done")
        except:
            spinner.fail("Get data from API failed")
            if debug:
                handle()
            return
        max_time = len(results["matches"]) * 10
        print(f"maximum time: {max_time} seconds")
        camera_type_list = []
        for result in results["matches"]:
            if camera_type in result["data"]:
                camera_type_list.append(result)
        store = []
        cnt = 0
        for result in camera_type_list:
            url = f"http://{result['ip_str']}:{result['port']}"
            cnt += 1
            try:
                r = requests.get(url, timeout=5)
                if r.status_code == 200:
                    if not check_empty:
                        print(
                            url_scheme.format(ip=result["ip_str"], port=result["port"])
                        )
                    else:
                        is_empty = self.check_empty(check_empty_url.format(url=url))
                        if is_empty:
                            store.append(result)
                            print(
                                url_scheme.format(
                                    ip=result["ip_str"], port=result["port"]
                                )
                            )
                        else:
                            print("[red]webcam is empty[/red]")
                            spinner.close()
                            continue
                    if geoip:
                        country, region, hour, minute = self.locator.locate(result["ip_str"])
                        print(f":earth_asia:[green]{country} , {region} {hour}:{minute}[/green]")
                        store[-1]["country"] = country
                        store[-1]["region"] = region
                    if tag:
                        tags = self.tag_image(check_empty_url.format(url=url))
                        for t in tags:
                            print(f"|[green]{t}[/green]|", end=" ")
                        if len(tags) == 0:
                            print("[i green]no description[i green]", end="")
                        print()
                        store[-1]["tags"] = tags
                    spinner.close()
            except KeyboardInterrupt:
                print("[red]terminating...")
                break
            except:
                if debug:
                    handle()
                continue
        return store

    def scan_preset(self, preset, check, tag, loc,debug=False):
        if preset not in self.config:
            raise KeyError("The preset entered doesn't exist")
        for key in self.config[preset]:
            if self.config[preset][key] == "[def]":
                self.config[preset][key] = self.config["default"][key]
        config = self.config[preset]
        config["check_empty"] = check
        config["tag"] = tag
        config["geoip"] = loc
        config['debug'] = debug
        self.scan(**config)
